# chanlimit.tcl for netbots.tcl v3.15
# designed to work with eggdrop 1.5.2 or higher
# Johoho's Eggdrop Page - http://johoho.eggdrop.cx
# chanlimit.tcl copyright (c) 1999,2000 by johoho <johoho@hojo-net.de>
# netbots.tcl copyright (c) 1998-2000 by slennox <slennox@egghelp.org>

## chanlimit.tcl component script v3.15.0+johoho ##
# script is a modification of slennox's chanlimit.tcl component v.3.15

proc cl_helpidx {hand chan idx} {
  if {![matchattr $hand m|m $chan]} {return 0}
  putidx $idx "chanlimit.tcl commands"
  putidx $idx " For channel masters:"
  putidx $idx "  dolimit       stoplimit     startlimit"
  putidx $idx " "
  return 1
}

proc cl_help {hand chan idx cmd} {
  if {[matchattr $hand m|m $chan]} {
    switch -exact -- $cmd {
      "dolimit" {
        putidx $idx "# dolimit"
        putwrap $idx 3 "Runs limit checking on demand."
        return 1
      }
      "stoplimit" {
        putidx $idx "# stoplimit"
        putwrap $idx 3 "Temporarily stops limiting (until the next rehash/restart)."
        return 1
      }
      "startlimit" {
        putidx $idx "# startlimit"
        putwrap $idx 3 "Restarts limiting."
        return 1
      }
    }
  }
  return 0
}

lappend nb_helpidx "cl_helpidx"
set nb_help(dolimit) "cl_help"
set nb_help(stoplimit) "cl_help"
set nb_help(startlimit) "cl_help"

proc cl_dolimit {} {
  global cl_bots cl_chans cl_echans cl_grace cl_limit cl_log cl_timer
  foreach chan [channels] {
    set ishere 0
	foreach thisbot $cl_bots	{
	  if {[handonchan $thisbot $chan] && [isop $thisbot $chan]} {
	    set ishere 1
	  }
	}
	if {!$ishere} {
      if {$cl_chans != ""} {
        if {[lsearch -exact $cl_chans [stl $chan]] == -1} {continue}
      }
      if {$cl_echans != ""} {
        if {[lsearch -exact $cl_echans [stl $chan]] != -1} {continue}
      }
      if {((![validchan $chan]) || (![botonchan $chan]) || (![botisop $chan]))} {continue}
      if {[string match *l* [lindex [lindex [channel info $chan] 0] 0]]} {continue}
      set newlimit [expr [llength [chanlist $chan]] + $cl_limit]
      if {[string match *l* [lindex [getchanmode $chan] 0]]} {
        set currlimit [string range [getchanmode $chan] [expr [string last " " [getchanmode $chan]] + 1] end]
      } else {
        set currlimit 0
      }
      if {$newlimit == $currlimit} {continue}
    if {$newlimit > $currlimit} {
        set difference [expr $newlimit - $currlimit]
      } elseif {$currlimit > $newlimit} {
        set difference [expr $currlimit - $newlimit]
      }
      if {$difference <= $cl_grace} {continue}
      pushmode $chan +l $newlimit
      if {$cl_log} {
        if {$cl_bots == ""} {
          putlog "chanlimit: set +l $newlimit on $chan"
        } else {
          putlog "chanlimit: set +l $newlimit on $chan ($cl_bots can't change the limit)"
        }
      }
    }
  }
  timer $cl_timer cl_dolimit
  return 0
}

proc cl_dccdolimit {hand idx arg} {
  nb_killtimer "cl_dolimit"
  putidx $idx "Checking limits.."
  cl_dolimit
  return 0
}

proc cl_dccstoplimit {hand idx arg} {
  if {[nb_killtimer "cl_dolimit"]} {
    putidx $idx "Limiting is now OFF."
  } else {
    putidx $idx "Limiting is already off."
  }
  return 0
}

proc cl_dccstartlimit {hand idx arg} {
  global cl_timer
  if {[string match *cl_dolimit* [timers]]} {
    putidx $idx "Limiting is already on."
  } else {
    timer $cl_timer cl_dolimit
    putidx $idx "Limiting is now ON."
  }
  return 0
}

set cl_chans [split [stl $cl_chans]] ; set cl_echans [split [stl $cl_echans]]

if {![string match *cl_dolimit* [timers]]} {
  timer $cl_timer cl_dolimit
}

bind dcc m|m dolimit cl_dccdolimit
bind dcc m|m stoplimit cl_dccstoplimit
bind dcc m|m startlimit cl_dccstartlimit

return
